# Демо-программа для взаимодействия со светодиодом.
# 1) Алгоритм демо-программы: светодиод горит красным (5 секунд) → мигает красным (5 секунд) → пульсирует красным (5 секунд)
# 2) затем горит зеленым (5 секунд) → мигает зеленым (5 секунд) → пульсирует зеленым (5 секунд)
# 3) затем горит синим (5 секунд) → мигает синим (5 секунд) → пульсирует синим (5 секунд)
import sys
from ctypes import *
from ctypes.util import find_library
import platform
import os

# Подключаем внешнюю библиотеку для работы с SDK
libName = ""
platform = platform.system()
if platform == "Windows":
    libName = "librisdk.dll"
if platform == "Linux":
    libName = "librisdk.so"

libPath = os.getenv("LIB_RISDK")
print(libPath)

lib = cdll.LoadLibrary(libPath + libName)

# Указываем типы аргументов для функций библиотеки RI_SDK
lib.RI_SDK_InitSDK.argtypes = [c_int, c_char_p]
lib.RI_SDK_CreateModelComponent.argtypes = [c_char_p, c_char_p, c_char_p, POINTER(c_int), c_char_p]
lib.RI_SDK_LinkPWMToController.argtypes = [c_int, c_int, c_uint8, c_char_p]
lib.RI_SDK_LinkLedToController.argtypes = [c_int, c_int, c_int, c_int, c_int, c_char_p]
lib.RI_SDK_DestroySDK.argtypes = [c_bool, c_char_p]
lib.RI_SDK_exec_RGB_LED_SinglePulse.argtypes = [c_int, c_int, c_int, c_int, c_int, c_bool, c_char_p]


def main():
    errTextC = create_string_buffer(1000)  # Текст ошибки. C type: char*
    i2c = c_int()
    pwm = c_int()
    led = c_int()

    # Инициализация библиотеки RI SDK с уровнем логирования 3
    errCode = lib.RI_SDK_InitSDK(3, errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)

    # Создание компонента i2c адаптера модели ch341
    errCode = lib.RI_SDK_CreateModelComponent("connector".encode(), "i2c_adapter".encode(), "ch341".encode(), i2c,
                                              errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)

    print("i2c: ", i2c.value)

    # Создание компонента ШИМ модели pca9685
    errCode = lib.RI_SDK_CreateModelComponent("connector".encode(), "pwm".encode(), "pca9685".encode(), pwm, errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)

    print("pwm: ", pwm.value)

    # Создание компонента светодиода модели ky016
    errCode = lib.RI_SDK_CreateModelComponent("executor".encode(), "led".encode(), "ky016".encode(), led, errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)

    print("led: ", led.value)

    # Связывание i2c с ШИМ
    errCode = lib.RI_SDK_LinkPWMToController(pwm, i2c, 0x40, errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)

    # Связывание ШИМ со светодиодом
    errCode = lib.RI_SDK_LinkLedToController(led, pwm, 15, 14, 13, errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)

    def ledLighting(r, g, b):
        # Непрерывное свечение светодиода заданным цветом в течение 5 секунд
        errCode = lib.RI_SDK_exec_RGB_LED_SinglePulse(led, r, g, b, 5000, False, errTextC)
        if errCode != 0:
            print(errCode, errTextC.raw.decode())
            sys.exit(2)

        # Мигание светодиода заданным цветом с продолжительностью 1 импульса в 500 миллисекунд и паузой между импульсами в 500 миллисекунд
        errCode = lib.RI_SDK_exec_RGB_LED_FlashingWithPause(led, r, g, b, 500, 500, 5, False, errTextC)
        if errCode != 0:
            print(errCode, errTextC.raw.decode())
            sys.exit(2)

        # Мерцания светодиода заданным цветом с продолжительностью 1 импульса в 1 секунду
        errCode = lib.RI_SDK_exec_RGB_LED_Flicker(led, r, g, b, 1000, 5, False, errTextC)
        if errCode != 0:
            print(errCode, errTextC.raw.decode())
            sys.exit(2)

    # запускаем свечение красным цветом
    ledLighting(255, 0, 0)
    # запускаем свечение зеленым цветом
    ledLighting(0, 255, 0)
    # запускаем свечение синим цветом
    ledLighting(0, 0, 255)

    # Удаление библиотеки со всеми компонентами
    errCode = lib.RI_SDK_DestroySDK(True, errTextC)
    if errCode != 0:
        print(errCode, errTextC.raw.decode())
        sys.exit(2)

    print("Success")


main()
